@@ -267,6 +267,10 @@ class BokehBackend(Plot):
         self._init_cyclers()
         super().__init__(*args, **kwargs)
 
+        # set labels
+        self._use_latex = kwargs.get("use_latex", cfg["bokeh"]["use_latex"])
+        self._set_labels()
+
         self._theme = kwargs.get("theme", cfg["bokeh"]["theme"])
         self._update_event = kwargs.get("update_event",
             cfg["bokeh"]["update_event"])
@@ -371,7 +375,7 @@ class BokehBackend(Plot):
                         else next(self._cm)
                     )
                     ds, line, cb, kw = self._create_gradient_line(
-                        x, y, param, colormap, s.label, s.rendering_kw)
+                        x, y, param, colormap, s.get_label(self._use_latex), s.rendering_kw)
                     self._fig.add_glyph(ds, line)
                     if self.legend:
                         self._handles[i] = cb
@@ -382,9 +386,13 @@ class BokehBackend(Plot):
                         source = {"xs": x, "ys": y, "us": param}
                     else:
                         x, y = s.get_data()
-                        source = {"xs": x, "ys": y}
+                        source = {
+                            "xs": x if not s.is_polar else y * np.cos(x),
+                            "ys": y if not s.is_polar else y * np.sin(x)
+                        }
 
-                    lkw = dict(line_width=2, legend_label=s.label,
+                    lkw = dict(line_width=2,
+                        legend_label=s.get_label(self._use_latex),
                         color=next(self._cl))
                     if not s.is_point:
                         kw = merge({}, lkw, s.rendering_kw)
@@ -397,6 +405,8 @@ class BokehBackend(Plot):
                         self._fig.circle("xs", "ys", source=source, **kw)
 
             elif s.is_contour and (not s.is_complex):
+                if s.is_polar:
+                    raise NotImplementedError()
                 x, y, z = s.get_data()
                 x, y, zz = [t.flatten() for t in [x, y, z]]
                 minx, miny, minz = min(x), min(y), min(zz)
@@ -417,7 +427,7 @@ class BokehBackend(Plot):
 
                 colormapper = self.bokeh.models.LinearColorMapper(
                     palette=cm, low=minz, high=maxz)
-                cbkw = dict(width=8, title=s.label)
+                cbkw = dict(width=8, title=s.get_label(self._use_latex))
                 colorbar = self.bokeh.models.ColorBar(
                     color_mapper=colormapper, **cbkw)
                 self._fig.add_layout(colorbar, "right")
@@ -449,14 +459,14 @@ class BokehBackend(Plot):
                         else next(self._cl)
                     )
                     source = self.bokeh.models.ColumnDataSource(data=data)
-                    qkw = dict(line_color=line_color, line_width=1, name=s.label)
+                    qkw = dict(line_color=line_color, line_width=1, name=s.get_label(self._use_latex))
                     kw = merge({}, qkw, quiver_kw)
                     glyph = self.bokeh.models.Segment(
                         x0="x0", y0="y0", x1="x1", y1="y1", **kw)
                     self._fig.add_glyph(source, glyph)
                     if isinstance(line_color, dict):
                         colorbar = self.bokeh.models.ColorBar(
-                            color_mapper=color_mapper, width=8, title=s.label)
+                            color_mapper=color_mapper, width=8, title=s.get_label(self._use_latex))
                         self._fig.add_layout(colorbar, "right")
                         self._handles[i] = colorbar
 
@@ -573,6 +583,8 @@ class BokehBackend(Plot):
         return data_source, glyph, colorbar, kw
 
     def _update_interactive(self, params):
+        np = import_module('numpy')
+
         rend = self.fig.renderers
         if len(rend) != len(self.series):
             self._process_series(self.series)
@@ -595,7 +607,10 @@ class BokehBackend(Plot):
                         source = {"xs": x, "ys": y, "us": param}
                     else:
                         x, y = self.series[i].get_data()
-                        source = {"xs": x, "ys": y}
+                        source = {
+                            "xs": x if not s.is_polar else y * np.cos(x),
+                            "ys": y if not s.is_polar else y * np.sin(x)
+                        }
                     rend[i].data_source.data.update(source)
 
                 elif s.is_contour and (not s.is_complex):
