@@ -21,8 +21,17 @@ class K3DBackend(Plot):
 
     quiver_kw : dict, optional
         A dictionary to customize the apppearance of quivers. Default to:
-        ``quiver_kw = dict(scale = 1)``.
-        Set `use_cm=False` to switch to a solid color.
+        ``quiver_kw = dict(scale = 1, pivot = "mid")``. The keys to this
+        dictionary are:
+
+        * ``scale``: a float number acting as a scale multiplier.
+        * ``pivot``: indicates the part of the arrow that is anchored to the
+          X, Y, Z grid. It can be ``"tail", "mid", "middle", "head"``.
+        * ``color``: set a solid color by specifying an integer color. If this
+          key is not provided, a default color or colormap is used, depenging
+          on the value of ``use_cm``.
+
+        Set ``use_cm=False`` to switch to a default solid color.
 
     show_label : boolean, optional
         Show/hide labels of the expressions. Default to False (labels not
@@ -35,6 +44,15 @@ class K3DBackend(Plot):
         Refer to k3d.line for more options.
         Set `use_cm=False` to switch to a solid color.
 
+    surface_kw : dict, optional
+        A dictionary of keywords/values to to customize the appearance which
+        is passed to:
+
+        - k3d.mesh function for 3D surface and parametric surface plots.
+          Refer to k3d.meshfor more options.
+        - k3d.marching_cubes function for 3D implicit plots. Refer to
+          k3d.marching_cubes for more options.
+
     use_cm : boolean, optional
         If True, apply a color map to the meshes/surface. If False, solid
         colors will be used instead. Default to True.
@@ -70,6 +88,9 @@ class K3DBackend(Plot):
     colormaps = []
     cyclic_colormaps = []
 
+    # quivers-pivot offsets
+    _qp_offset = {"tail": 0, "mid": 0.5, "middle": 0.5, "tip": 1}
+
     def __new__(cls, *args, **kwargs):
         return object.__new__(cls)
 
@@ -106,6 +127,8 @@ class K3DBackend(Plot):
         if self._get_mode() != 0:
             raise ValueError(
                 "Sorry, K3D backend only works within Jupyter Notebook")
+        self._use_latex = kwargs.get("use_latex", cfg["k3d"]["use_latex"])
+        self._set_labels("%s")
 
         self._show_label = kwargs.get("show_label", False)
         self._bounds = []
@@ -208,16 +231,11 @@ class K3DBackend(Plot):
 
             elif s.is_3Dline:
                 x, y, z, param = s.get_data()
-                # K3D doesn't like masked arrays, so filled them with NaN
-                x, y, z = [
-                    np.ma.filled(t) if isinstance(t, np.ma.core.MaskedArray) else t
-                    for t in [x, y, z]
-                ]
                 vertices = np.vstack([x, y, z]).T.astype(np.float32)
                 # keyword arguments for the line object
                 a = dict(
                     width=0.1,
-                    name=s.label if self._show_label else None,
+                    name=self._get_series_label(s, "%s") if self._show_label else None,
                     color=self._convert_to_int(next(self._cl)),
                     shader="mesh",
                 )
@@ -229,29 +247,24 @@ class K3DBackend(Plot):
                 line = self.k3d.line(vertices, **kw)
                 self._fig += line
 
-            elif (s.is_3Dsurface and not s.is_domain_coloring):
-                x, y, z = s.get_data()
-                # K3D doesn't like masked arrays, so filled them with NaN
-                x, y, z = [
-                    np.ma.filled(t) if isinstance(t, np.ma.core.MaskedArray) else t
-                    for t in [x, y, z]
-                ]
-
-                # TODO:
-                # Can I use get_vertices_indices also for non parametric surfaces?
+            elif (s.is_3Dsurface and (not s.is_domain_coloring) and (not s.is_implicit)):
                 if s.is_parametric:
+                    x, y, z, u, v = s.get_data()
                     vertices, indices = get_vertices_indices(x, y, z)
                     vertices = vertices.astype(np.float32)
+                    attribute = s.color_func(vertices[:, 0], vertices[:, 1], vertices[:, 2], u.flatten().astype(np.float32), v.flatten().astype(np.float32))
                 else:
+                    x, y, z = s.get_data()
                     x = x.flatten()
                     y = y.flatten()
                     z = z.flatten()
                     vertices = np.vstack([x, y, z]).T.astype(np.float32)
                     indices = Triangulation(x, y).triangles.astype(np.uint32)
+                    attribute = s.color_func(vertices[:, 0], vertices[:, 1], vertices[:, 2])
 
                 self._high_aspect_ratio(x, y, z)
                 a = dict(
-                    name=s.label if self._show_label else None,
+                    name=s.get_label(self._use_latex, "%s") if self._show_label else None,
                     side="double",
                     flat_shading=False,
                     wireframe=False,
@@ -259,19 +272,34 @@ class K3DBackend(Plot):
                 )
                 if s.use_cm:
                     a["color_map"] = next(self._cm)
-                    a["attribute"] = z.astype(np.float32)
+                    a["attribute"] = attribute
+                    a["color_range"] = [attribute.min(), attribute.max()]
+
                 kw = merge({}, a, s.rendering_kw)
                 surf = self.k3d.mesh(vertices, indices, **kw)
 
                 self._fig += surf
 
+            elif s.is_implicit and s.is_3Dsurface:
+                _, _, _, r = s.get_data()
+                xmin, xmax = s.start_x, s.end_x
+                ymin, ymax = s.start_y, s.end_y
+                zmin, zmax = s.start_z, s.end_z
+                a = dict(
+                    xmin=xmin, xmax=xmax,
+                    ymin=ymin, ymax=ymax,
+                    zmin=zmin, zmax=zmax,
+                    compression_level=9,
+                    level=0.0, flat_shading=True,
+                    color=self._convert_to_int(next(self._cl))
+                )
+                kw = merge({}, a, s.rendering_kw)
+                plt_iso = self.k3d.marching_cubes(r.astype(np.float32), **kw)
+
+                self._fig += plt_iso
+
             elif s.is_3Dvector and s.is_streamlines:
                 xx, yy, zz, uu, vv, ww = s.get_data()
-                # K3D doesn't like masked arrays, so filled them with NaN
-                xx, yy, zz, uu, vv, ww = [
-                    np.ma.filled(t) if isinstance(t, np.ma.core.MaskedArray)
-                    else t for t in [xx, yy, zz, uu, vv, ww]]
-
                 vertices, magn = compute_streamtubes(
                     xx, yy, zz, uu, vv, ww, s.rendering_kw)
 
@@ -293,44 +321,38 @@ class K3DBackend(Plot):
 
             elif s.is_3Dvector:
                 xx, yy, zz, uu, vv, ww = s.get_data()
-                # K3D doesn't like masked arrays, so filled them with NaN
-                xx, yy, zz, uu, vv, ww = [
-                    np.ma.filled(t) if isinstance(t, np.ma.core.MaskedArray) else t
-                    for t in [xx, yy, zz, uu, vv, ww]
-                ]
-                xx, yy, zz, uu, vv, ww = [
-                    t.flatten().astype(np.float32) for t in [xx, yy, zz, uu, vv, ww]
-                ]
                 qkw = dict(scale=1)
                 qkw = merge(qkw, s.rendering_kw)
-                scale = qkw["scale"]
-                magnitude = np.sqrt(uu ** 2 + vv ** 2 + ww ** 2)
-                vectors = np.array((uu, vv, ww)).T * scale
-                origins = np.array((xx, yy, zz)).T
                 quiver_kw = s.rendering_kw
                 if s.use_cm and ("color" not in quiver_kw.keys()):
                     colormap = next(self._cm)
-                    colors = self.k3d.helpers.map_colors(
-                        magnitude, colormap, [])
+                    # store useful info for interactive vector plots
                     self._handles[ii] = [qkw, colormap]
                 else:
+                    colormap = None
                     col = quiver_kw.get("color", next(self._cl))
                     if not isinstance(col, int):
                         col = self._convert_to_int(col)
-                    colors = col * np.ones(len(magnitude))
-                    self._handles[ii] = [qkw, None]
-                vec_colors = np.zeros(2 * len(colors))
-                for i, c in enumerate(colors):
-                    vec_colors[2 * i] = c
-                    vec_colors[2 * i + 1] = c
-                vec_colors = vec_colors.astype(np.uint32)
+                    solid_color = col * np.ones(xx.size)
+                    self._handles[ii] = [qkw, colormap]
+
+                origins, vectors, colors = self._build_k3d_vector_data(xx, yy, zz, uu, vv, ww, qkw, colormap)
+                if colors is None:
+                    colors = solid_color
+                vec_colors = self._create_vector_colors(colors)
+
+                pivot = quiver_kw.get("pivot", "mid")
+                if pivot not in self._qp_offset.keys():
+                    raise ValueError(
+                        "`pivot` must be one of the following values: "
+                        "{}".format(list(self._qp_offset.keys())))
 
                 vec_kw = qkw.copy()
-                kw_to_remove = ["scale", "color"]
+                kw_to_remove = ["scale", "color", "pivot"]
                 for k in kw_to_remove:
                     if k in vec_kw.keys():
                         vec_kw.pop(k)
-                vec_kw["origins"] = origins - vectors / 2
+                vec_kw["origins"] = origins - vectors * self._qp_offset[pivot]
                 vec_kw["vectors"] = vectors
                 vec_kw["colors"] = vec_colors
 
@@ -346,7 +368,7 @@ class K3DBackend(Plot):
                 self._high_aspect_ratio(x, y, z)
 
                 a = dict(
-                    name=s.label if self._show_label else None,
+                    name=s.get_label(self._use_latex, "%s") if self._show_label else None,
                     side="double",
                     flat_shading=False,
                     wireframe=False,
@@ -388,6 +410,39 @@ class K3DBackend(Plot):
             )
         self._fig.auto_rendering = True
 
+    def _build_k3d_vector_data(self, xx, yy, zz, uu, vv, ww, qkw, colormap):
+        """Assemble the origins, vectors and colors (if possible) matrices.
+        """
+        np = import_module('numpy')
+
+        xx, yy, zz, uu, vv, ww = [
+            t.flatten().astype(np.float32) for t in [xx, yy, zz, uu, vv, ww]
+        ]
+        scale = qkw["scale"]
+        magnitude = np.sqrt(uu ** 2 + vv ** 2 + ww ** 2)
+        vectors = np.array((uu, vv, ww)).T * scale
+        origins = np.array((xx, yy, zz)).T
+
+        colors = None
+        if colormap is not None:
+            colors = self.k3d.helpers.map_colors(
+                magnitude, colormap, [])
+
+        return origins, vectors, colors
+
+    def _create_vector_colors(self, colors):
+        """Create a color matrix. Each vector requires one color for the tail
+        and one for the head.
+        """
+        np = import_module('numpy')
+
+        vec_colors = np.zeros(2 * len(colors))
+        for i, c in enumerate(colors):
+            vec_colors[2 * i] = c
+            vec_colors[2 * i + 1] = c
+        vec_colors = vec_colors.astype(np.uint32)
+        return vec_colors
+
     def _high_aspect_ratio(self, x, y, z):
         """Look for high aspect ratio meshes, where (dz >> dx, dy) and
         eventually set the bounds around the mid point of the mesh in order
@@ -434,36 +489,35 @@ class K3DBackend(Plot):
                     vertices = np.vstack([x, y, z]).T.astype(np.float32)
                     self._fig.objects[i].vertices = vertices
 
-                elif s.is_3Dsurface and (not s.is_domain_coloring):
-                    x, y, z = self.series[i].get_data()
-                    x, y, z = [t.flatten().astype(np.float32) for t in [x, y, z]]
+                elif s.is_3Dsurface and (not s.is_domain_coloring) and (not s.is_implicit):
+                    if s.is_parametric:
+                        x, y, z, u, v = s.get_data()
+                        x, y, z, u, v = [t.flatten().astype(np.float32) for t in [x, y, z, u, v]]
+                        attribute = s.color_func(x, y, z, u, v)
+                    else:
+                        x, y, z = s.get_data()
+                        x, y, z = [t.flatten().astype(np.float32) for t in [x, y, z]]
+                        attribute = s.color_func(x, y, z)
+
                     vertices = np.vstack([x, y, z]).astype(np.float32)
                     self._fig.objects[i].vertices = vertices.T
-                    self._fig.objects[i].attribute = z
-                    self._fig.objects[i].color_range = [z.min(), z.max()]
+                    if s.use_cm:
+                        self._fig.objects[i].attribute = attribute
+                        self._fig.objects[i].color_range = [attribute.min(), attribute.max()]
 
                 elif s.is_vector and s.is_3D:
                     if s.is_streamlines:
                         raise NotImplementedError
 
                     xx, yy, zz, uu, vv, ww = self.series[i].get_data()
-                    xx, yy, zz, uu, vv, ww = [
-                        t.flatten().astype(np.float32) for t in [xx, yy, zz, uu, vv, ww]
-                    ]
-                    magnitude = np.sqrt(uu ** 2 + vv ** 2 + ww ** 2)
                     qkw, colormap = self._handles[i]
-                    if colormap is not None:
-                        colors = self.k3d.helpers.map_colors(
-                            magnitude, colormap, [])
-                        vec_colors = np.zeros(2 * len(colors))
-                        for j, c in enumerate(colors):
-                            vec_colors[2 * j] = c
-                            vec_colors[2 * j + 1] = c
-                        vec_colors = vec_colors.astype(np.uint32)
+                    origins, vectors, colors = self._build_k3d_vector_data(xx, yy, zz, uu, vv, ww, qkw, colormap)
+                    if colors is not None:
+                        vec_colors = self._create_vector_colors(colors)
                         self.fig.objects[i].colors = vec_colors
 
-                    scale = qkw["scale"]
-                    vectors = np.array((uu, vv, ww)).T * scale
+                    pivot = s.rendering_kw.get("pivot", "mid")
+                    self.fig.objects[i].origins = origins - vectors * self._qp_offset[pivot]
                     self.fig.objects[i].vectors = vectors
 
                 elif s.is_complex and s.is_3Dsurface:
