@@ -1,6 +1,7 @@
 from spb.defaults import cfg
 from spb.backends.base_backend import Plot
 from spb.backends.utils import compute_streamtubes
+from sympy import latex
 from sympy.external import import_module
 import itertools
 
@@ -147,11 +148,17 @@ class MatplotlibBackend(Plot):
             cm.viridis, cm.autumn, cm.winter, cm.plasma, cm.jet,
             cm.gnuplot, cm.brg, cm.coolwarm, cm.cool, cm.summer]
         self.cyclic_colormaps = [cm.twilight, cm.hsv]
-        self.colorloop = cm.tab10.colors
+        # load default colorloop
+        self.colorloop = self.plt.rcParams['axes.prop_cycle'].by_key()["color"]
+
 
         self._init_cyclers()
         super().__init__(*args, **kwargs)
 
+        # set labels
+        self._use_latex = kwargs.get("use_latex", cfg["matplotlib"]["use_latex"])
+        self._set_labels()
+
         if ((len([s for s in self._series if s.is_2Dline]) > 10) and
             (not type(self).colorloop) and
             not ("process_piecewise" in kwargs.keys())):
@@ -231,6 +238,8 @@ class MatplotlibBackend(Plot):
             kwargs = dict(aspect=aspect)
             if all(is_3D):
                 kwargs["projection"] = "3d"
+            elif any(s.is_2Dline and s.is_polar for s in self.series):
+                kwargs["projection"] = "polar"
             self.ax = self._fig.add_subplot(1, 1, 1, **kwargs)
 
     @property
@@ -268,7 +277,7 @@ class MatplotlibBackend(Plot):
         points = np.ma.array(points).T.reshape(-1, 1, dim)
         return np.ma.concatenate([points[:-1], points[1:]], axis=1)
 
-    def _add_colorbar(self, c, label, use_cm, override=False):
+    def _add_colorbar(self, c, label, use_cm, override=False, norm=None, cmap=None):
         """Add a colorbar for the specificied collection
 
         Parameters
@@ -285,12 +294,18 @@ class MatplotlibBackend(Plot):
             Hence, to show it we set override=True.
             Default to False.
         """
+        np = import_module('numpy')
+
         # design choice: instead of showing a legend entry (which
         # would require to work with proxy artists and custom
         # classes in order to create a gradient line), just show a
         # colorbar with the name of the expression on the side.
         if (self.legend and use_cm) or override:
-            cb = self._fig.colorbar(c, ax=self.ax)
+            if norm is None:
+                cb = self._fig.colorbar(c, ax=self.ax)
+            else:
+                mappable = self.cm.ScalarMappable(cmap=cmap, norm=norm)
+                cb = self._fig.colorbar(mappable)
             cb.set_label(label, rotation=90)
             return True
         return False
@@ -340,14 +355,14 @@ class MatplotlibBackend(Plot):
                     segments = self.get_segments(x, y)
                     c = self.LineCollection(segments, **kw)
                     self.ax.add_collection(c)
-                    is_cb_added = self._add_colorbar(c, s.label, s.use_cm)
+                    is_cb_added = self._add_colorbar(c, s.get_label(self._use_latex), s.use_cm)
                     self._add_handle(i, c, kw, is_cb_added, self._fig.axes[-1])
                 else:
                     if s.is_parametric:
                         x, y, param = s.get_data()
                     else:
                         x, y = s.get_data()
-                    lkw = dict(label=s.label, color=next(self._cl))
+                    lkw = dict(label=s.get_label(self._use_latex), color=next(self._cl))
                     if s.is_point:
                         lkw["marker"] = "o"
                         lkw["linestyle"] = "None"
@@ -362,7 +377,7 @@ class MatplotlibBackend(Plot):
                 ckw = dict(cmap=next(self._cm))
                 kw = merge({}, ckw, s.rendering_kw)
                 c = self.ax.contourf(x, y, z, **kw)
-                self._add_colorbar(c, s.label, s.use_cm, True)
+                self._add_colorbar(c, s.get_label(self._use_latex), s.use_cm, True)
                 self._add_handle(i, c, kw, self._fig.axes[-1])
 
             elif s.is_3Dline:
@@ -377,16 +392,16 @@ class MatplotlibBackend(Plot):
                         kw = merge({}, lkw, s.rendering_kw)
                         c = Line3DCollection(segments, **kw)
                         self.ax.add_collection(c)
-                        self._add_colorbar(c, s.label, s.use_cm)
+                        self._add_colorbar(c, s.get_label(self._use_latex), s.use_cm)
                         self._add_handle(i, c)
                     else:
-                        lkw["label"] = s.label
+                        lkw["label"] = s.get_label(self._use_latex)
                         kw = merge({}, lkw, s.rendering_kw)
                         l = self.ax.plot(x, y, z, **kw)
                         self._add_handle(i, l)
                 else:
                     # 3D points
-                    lkw["label"] = s.label
+                    lkw["label"] = s.get_label(self._use_latex)
                     lkw["color"] = next(self._cl)
                     kw = merge({}, lkw, s.rendering_kw)
                     l = self.ax.scatter(x, y, z, **kw)
@@ -395,20 +410,36 @@ class MatplotlibBackend(Plot):
                 ylims.append((np.amin(y), np.amax(y)))
                 zlims.append((np.amin(z), np.amax(z)))
 
-            elif (s.is_3Dsurface and not s.is_domain_coloring):
-                x, y, z = s.get_data()
+            elif (s.is_3Dsurface and (not s.is_domain_coloring) and (not s.is_implicit)):
+                if not s.is_parametric:
+                    x, y, z = self.series[i].get_data()
+                    facecolors = s.color_func(x, y, z)
+                else:
+                    x, y, z, u, v = self.series[i].get_data()
+                    facecolors = s.color_func(x, y, z, u, v)
                 skw = dict(rstride=1, cstride=1, linewidth=0.1)
+                norm, cmap = None, None
                 if s.use_cm:
-                    skw["cmap"] = next(self._cm)
+                    norm = self.Normalize(vmin=np.amin(facecolors), vmax=np.amax(facecolors))
+                    cmap = next(self._cm)
+                    skw["cmap"] = cmap
+                else:
+                    skw["color"] = next(self._cl)
+
                 kw = merge({}, skw, s.rendering_kw)
+                if s.use_cm:
+                    # facecolors must be computed here because s.rendering_kw
+                    # might have its own cmap
+                    cmap = kw["cmap"]
+                    kw["facecolors"] = cmap(norm(facecolors))
                 c = self.ax.plot_surface(x, y, z, **kw)
-                is_cb_added = self._add_colorbar(c, s.label, s.use_cm)
+                is_cb_added = self._add_colorbar(c, s.get_label(self._use_latex), s.use_cm, norm=norm, cmap=cmap)
                 self._add_handle(i, c, kw, is_cb_added, self._fig.axes[-1])
                 xlims.append((np.amin(x), np.amax(x)))
                 ylims.append((np.amin(y), np.amax(y)))
                 zlims.append((np.amin(z), np.amax(z)))
 
-            elif s.is_implicit:
+            elif s.is_implicit and not s.is_3Dsurface:
                 points = s.get_data()
                 if len(points) == 2:
                     # interval math plotting
@@ -446,7 +477,7 @@ class MatplotlibBackend(Plot):
                             kw = merge({}, skw, s.rendering_kw)
                             sp = self.ax.streamplot(xx, yy, uu, vv, **kw)
                             is_cb_added = self._add_colorbar(
-                                sp.lines, s.label, s.use_cm)
+                                sp.lines, s.get_label(self._use_latex), s.use_cm)
                         else:
                             skw["color"] = next(self._cl)
                             kw = merge({}, skw, s.rendering_kw)
@@ -463,7 +494,7 @@ class MatplotlibBackend(Plot):
                             kw = merge({}, qkw, s.rendering_kw)
                             q = self.ax.quiver(xx, yy, uu, vv, magn, **kw)
                             is_cb_added = self._add_colorbar(
-                                q, s.label, s.use_cm)
+                                q, s.get_label(self._use_latex), s.use_cm)
                         else:
                             is_cb_added = False
                             qkw["color"] = next(self._cl)
@@ -494,10 +525,10 @@ class MatplotlibBackend(Plot):
                             kw = merge({}, lkw, stream_kw)
                             c = Line3DCollection(segments, **kw)
                             self.ax.add_collection(c)
-                            self._add_colorbar(c, s.label, s.use_cm)
+                            self._add_colorbar(c, s.get_label(self._use_latex), s.use_cm)
                             self._add_handle(i, c)
                         else:
-                            lkw["label"] = s.label
+                            lkw["label"] = s.get_label(self._use_latex)
                             kw = merge({}, lkw, stream_kw)
                             l = self.ax.plot(vertices[:, 0], vertices[:, 1],
                                 vertices[:, 2], **kw)
@@ -514,7 +545,7 @@ class MatplotlibBackend(Plot):
                             kw = merge({}, qkw, s.rendering_kw)
                             q = self.ax.quiver(xx, yy, zz, uu, vv, ww, **kw)
                             is_cb_added = self._add_colorbar(
-                                q, s.label, s.use_cm)
+                                q, s.get_label(self._use_latex), s.use_cm)
                         else:
                             qkw["color"] = next(self._cl)
                             kw = merge({}, qkw, s.rendering_kw)
@@ -745,15 +776,16 @@ class MatplotlibBackend(Plot):
         if self.zlim:
             self.ax.set_zlim(self.zlim)
 
-    def _update_colorbar(self, cax, param, kw, label):
+    def _update_colorbar(self, cax, cmap, label, param=None, norm=None):
         """This method reduces code repetition.
         The name is misleading: here we create a new colorbar which will be
         placed on the same colorbar axis as the original.
         """
         np = import_module('numpy')
         cax.clear()
-        norm = self.Normalize(vmin=np.amin(param), vmax=np.amax(param))
-        mappable = self.cm.ScalarMappable(cmap=kw["cmap"], norm=norm)
+        if norm is None:
+            norm = self.Normalize(vmin=np.amin(param), vmax=np.amax(param))
+        mappable = self.cm.ScalarMappable(cmap=cmap, norm=norm)
         self._fig.colorbar(mappable, orientation="vertical", label=label, cax=cax)
 
     def _update_interactive(self, params):
@@ -782,7 +814,8 @@ class MatplotlibBackend(Plot):
                         self._handles[i][0].set_array(param)
                         kw, is_cb_added, cax = self._handles[i][1:]
                         if is_cb_added:
-                            self._update_colorbar(cax, param, kw, s.label)
+                            norm = self.Normalize(vmin=np.amin(param), vmax=np.amax(param))
+                            self._update_colorbar(cax, kw["cmap"], s.get_label(self._use_latex), norm=norm)
                         xlims.append((np.amin(x), np.amax(x)))
                         ylims.append((np.amin(y), np.amax(y)))
                     else:
@@ -815,26 +848,35 @@ class MatplotlibBackend(Plot):
                     for c in self._handles[i][0].collections:
                         c.remove()
                     self._handles[i][0] = self.ax.contourf(x, y, z, **kw)
-                    self._update_colorbar(cax, z, kw, s.label)
+                    self._update_colorbar(cax, kw["cmap"], s.get_label(self._use_latex), param=z)
                     xlims.append((np.amin(x), np.amax(x)))
                     ylims.append((np.amin(y), np.amax(y)))
 
-                elif s.is_3Dsurface and (not s.is_domain_coloring):
-                    x, y, z = self.series[i].get_data()
+                elif s.is_3Dsurface and (not s.is_domain_coloring) and (not s.is_implicit):
+                    if not s.is_parametric:
+                        x, y, z = self.series[i].get_data()
+                        facecolors = s.color_func(x, y, z)
+                    else:
+                        x, y, z, u, v = self.series[i].get_data()
+                        facecolors = s.color_func(x, y, z, u, v)
                     # TODO: by setting the keyword arguments, somehow the
                     # update becomes really really slow.
                     kw, is_cb_added, cax = self._handles[i][1:]
+
+                    if is_cb_added:
+                        norm = self.Normalize(vmin=np.amin(facecolors), vmax=np.amax(facecolors))
+                        kw["facecolors"] = kw["cmap"](norm(facecolors))
                     self._handles[i][0].remove()
                     self._handles[i][0] = self.ax.plot_surface(
                         x, y, z, **kw)
 
                     if is_cb_added:
-                        self._update_colorbar(cax, z, kw, s.label)
+                        self._update_colorbar(cax, kw["cmap"], s.get_label(self._use_latex), norm=norm)
                     xlims.append((np.amin(x), np.amax(x)))
                     ylims.append((np.amin(y), np.amax(y)))
                     zlims.append((np.amin(z), np.amax(z)))
 
-                elif s.is_implicit:
+                elif s.is_implicit and not s.is_3Dsurface:
                     points = s.get_data()
                     if len(points) == 2:
                         raise NotImplementedError
@@ -863,7 +905,7 @@ class MatplotlibBackend(Plot):
 
                     if is_cb_added:
                         magn = np.sqrt(uu ** 2 + vv ** 2 + ww ** 2)
-                        self._update_colorbar(cax, magn, kw, s.label)
+                        self._update_colorbar(cax, kw["cmap"], s.get_label(self._use_latex), param=magn)
                     xlims.append((np.amin(xx), np.amax(xx)))
                     ylims.append((np.amin(yy), np.amax(yy)))
                     zlims.append((np.nanmin(zz), np.nanmax(zz)))
@@ -886,7 +928,7 @@ class MatplotlibBackend(Plot):
 
                         if is_cb_added:
                             self._handles[i][0].set_UVC(uu, vv, magn)
-                            self._update_colorbar(cax, magn, kw, s.label)
+                            self._update_colorbar(cax, kw["cmap"], s.get_label(self._use_latex), magn)
                         else:
                             self._handles[i][0].set_UVC(uu, vv)
                     xlims.append((np.amin(xx), np.amax(xx)))
