@@ -42,9 +42,13 @@ class PlotlyBackend(Plot):
           Refer [#fn6]_ for more options.
 
     surface_kw : dict, optional
-        A dictionary of keywords/values which is passed to Plotly's
-        Surface function to customize the appearance.
-        Refer to [#fn7]_ for more options.
+        A dictionary of keywords/values to to customize the appearance which
+        is passed to:
+
+        - Plotly's Surface function for 3D surface and parametric surface
+          plots. Refer to [#fn7]_ for more options.
+        - Plotly's Isosurface function for 3D implicit plots. Refer to [#fn14]_
+          for more options.
 
     stream_kw : dict, optional
         A dictionary of keywords/values which is passed to Plotly's
@@ -76,6 +80,7 @@ class PlotlyBackend(Plot):
     .. [#fn9] https://plotly.com/python/streamtube-plot/
     .. [#fn10] https://plotly.com/python/templates/
     .. [#fn13] https://github.com/plotly/plotly.js/issues/5003
+    .. [#fn14] https://plotly.com/python/3d-isosurface-plots/
 
 
     Notes
@@ -137,6 +142,11 @@ class PlotlyBackend(Plot):
         self._init_cyclers()
         super().__init__(*args, **kwargs)
 
+        # NOTE: Plotly 3D currently doesn't support latex labels
+        # https://github.com/plotly/plotly.js/issues/608
+        self._use_latex = kwargs.get("use_latex", cfg["plotly"]["use_latex"])
+        self._set_labels()
+
         if ((len([s for s in self._series if s.is_2Dline]) > 10) and
             (not type(self).colorloop) and
             not ("process_piecewise" in kwargs.keys())):
@@ -257,7 +267,7 @@ class PlotlyBackend(Plot):
                     if (not s.is_point) and (not s.use_cm):
                         mode = "lines"
                     lkw = dict(
-                        name=s.label,
+                        name=s.get_label(self._use_latex),
                         line_color=next(self._cl),
                         mode=mode,
                         marker=dict(
@@ -269,7 +279,7 @@ class PlotlyBackend(Plot):
                             ),
                             size=6,
                             showscale=self.legend and s.use_cm,
-                            colorbar=self._create_colorbar(ii, s.label, True),
+                            colorbar=self._create_colorbar(ii, s.get_label(self._use_latex), True),
                         ),
                         customdata=param,
                         hovertemplate=(
@@ -284,7 +294,7 @@ class PlotlyBackend(Plot):
                     x, y = s.get_data()
                     color = next(self._cl)
                     lkw = dict(
-                        name=s.label,
+                        name=s.get_label(self._use_latex),
                         mode="lines" if not s.is_point else "markers",
                         line_color=color
                     )
@@ -300,7 +310,11 @@ class PlotlyBackend(Plot):
                                 )
                             )
                     kw = merge({}, lkw, s.rendering_kw)
-                    self._fig.add_trace(go.Scatter(x=x, y=y, **kw))
+                    if s.is_polar:
+                        kw.setdefault("thetaunit", "radians")
+                        self._fig.add_trace(go.Scatterpolar(r=y, theta=x, **kw))
+                    else:
+                        self._fig.add_trace(go.Scatter(x=x, y=y, **kw))
             elif s.is_3Dline:
                 # NOTE: As a design choice, I decided to show the legend entry
                 # as well as the colorbar (if use_cm=True). Even though the
@@ -310,7 +324,7 @@ class PlotlyBackend(Plot):
                 x, y, z, param = s.get_data()
                 if not s.is_point:
                     lkw = dict(
-                        name=s.label,
+                        name=s.get_label(self._use_latex),
                         mode="lines",
                         line=dict(
                             width=4,
@@ -321,29 +335,37 @@ class PlotlyBackend(Plot):
                             ),
                             color=param,
                             showscale=self.legend and s.use_cm,
-                            colorbar=self._create_colorbar(ii, s.label, True),
+                            colorbar=self._create_colorbar(ii, s.get_label(self._use_latex), True),
                         ),
                     )
                 else:
                     lkw = dict(
-                        name=s.label,
+                        name=s.get_label(self._use_latex),
                         mode="markers",
                         line_color=next(self._cl))
                 kw = merge({}, lkw, s.rendering_kw)
                 self._fig.add_trace(go.Scatter3d(x=x, y=y, z=z, **kw))
 
-            elif (s.is_3Dsurface and not s.is_domain_coloring):
-                xx, yy, zz = s.get_data()
+            elif s.is_3Dsurface and (not s.is_domain_coloring) and (not s.is_implicit):
+                if not s.is_parametric:
+                    xx, yy, zz = s.get_data()
+                    surfacecolor = s.color_func(xx, yy, zz)
+                else:
+                    xx, yy, zz, uu, vv = s.get_data()
+                    surfacecolor = s.color_func(xx, yy, zz, uu, vv)
 
                 # create a solid color to be used when s.use_cm=False
                 col = next(self._cl)
                 colorscale = [[0, col], [1, col]]
                 colormap = next(self._cm)
                 skw = dict(
-                    name=s.label,
+                    name=s.get_label(self._use_latex),
                     showscale=self.legend and show_3D_colorscales,
-                    colorbar=self._create_colorbar(ii, s.label),
+                    colorbar=self._create_colorbar(ii, s.get_label(self._use_latex)),
                     colorscale=colormap if s.use_cm else colorscale,
+                    surfacecolor=surfacecolor,
+                    cmin=surfacecolor.min(),
+                    cmax=surfacecolor.max()
                 )
 
                 kw = merge({}, skw, s.rendering_kw)
@@ -351,7 +373,30 @@ class PlotlyBackend(Plot):
 
                 count += 1
 
+            elif s.is_3Dsurface and s.is_implicit:
+                xx, yy, zz, rr = s.get_data()
+                # create a solid color
+                col = next(self._cl)
+                colorscale = [[0, col], [1, col]]
+                skw = dict(
+                    isomin=0,
+                    isomax=0,
+                    showscale=False,
+                    colorscale=colorscale
+                )
+                kw = merge({}, skw, s.rendering_kw)
+                self._fig.add_trace(go.Isosurface(
+                    x=xx.flatten(),
+                    y=yy.flatten(),
+                    z=zz.flatten(),
+                    value=rr.flatten(), **kw
+                ))
+                count += 1
+
+
             elif s.is_contour and (not s.is_complex):
+                if s.is_polar:
+                    raise NotImplementedError()
                 xx, yy, zz = s.get_data()
                 xx = xx[0, :]
                 yy = yy[:, 0]
@@ -361,7 +406,7 @@ class PlotlyBackend(Plot):
                         showlabels=False,
                     ),
                     colorscale=next(self._cm),
-                    colorbar=self._create_colorbar(ii, s.label, show_2D_vectors),
+                    colorbar=self._create_colorbar(ii, s.get_label(self._use_latex), show_2D_vectors),
                 )
                 kw = merge({}, ckw, s.rendering_kw)
                 self._fig.add_trace(go.Contour(x=xx, y=yy, z=zz, **kw))
@@ -375,14 +420,14 @@ class PlotlyBackend(Plot):
                     # https://community.plotly.com/t/how-to-make-python-quiver-with-colorscale/41028
                     if s.is_streamlines:
                         skw = dict(
-                            line_color=next(self._qc), arrow_scale=0.15, name=s.label
+                            line_color=next(self._qc), arrow_scale=0.15, name=s.get_label(self._use_latex)
                         )
                         kw = merge({}, skw, s.rendering_kw)
                         stream = create_streamline(
                             xx[0, :], yy[:, 0], uu, vv, **kw)
                         self._fig.add_trace(stream.data[0])
                     else:
-                        qkw = dict(line_color=next(self._qc), scale=0.075, name=s.label)
+                        qkw = dict(line_color=next(self._qc), scale=0.075, name=s.get_label(self._use_latex))
                         kw = merge({}, qkw, s.rendering_kw)
                         quiver = create_quiver(xx, yy, uu, vv, **kw)
                         self._fig.add_trace(quiver.data[0])
@@ -401,7 +446,7 @@ class PlotlyBackend(Plot):
                             ),
                             sizeref=0.3,
                             showscale=self.legend and s.use_cm,
-                            colorbar=self._create_colorbar(ii, s.label),
+                            colorbar=self._create_colorbar(ii, s.get_label(self._use_latex)),
                             starts=dict(
                                 x=seeds_points[:, 0],
                                 y=seeds_points[:, 1],
@@ -431,7 +476,7 @@ class PlotlyBackend(Plot):
                             colorscale=next(self._cm),
                             sizemode="absolute",
                             sizeref=40,
-                            colorbar=self._create_colorbar(ii, s.label),
+                            colorbar=self._create_colorbar(ii, s.get_label(self._use_latex)),
                         )
                         kw = merge({}, qkw, s.rendering_kw)
                         self._fig.add_trace(
@@ -458,7 +503,7 @@ class PlotlyBackend(Plot):
                             dx=(xmax - xmin) / s.n1,
                             dy=(ymax - ymin) / s.n2,
                             z=img,
-                            name=s.label,
+                            name=s.get_label(self._use_latex),
                             customdata=np.dstack([mag, angle]),
                             hovertemplate=(
                                 "x: %{x}<br />y: %{y}<br />RGB: %{z}"
@@ -523,12 +568,11 @@ class PlotlyBackend(Plot):
                         for loc, c in zip(locations, colorscale):
                             tmp.append([loc, "rgb" + str(tuple(c))])
                         colorscale = tmp
-                        # print(colorscale)
                     else:
                         colorscale = [[0, col], [1, col]]
                     colormap = next(self._cyccm)
                     skw = dict(
-                        name=s.label,
+                        name=s.get_label(self._use_latex),
                         showscale=True,
                         colorbar=dict(
                             x=1 + 0.1 * count,
@@ -565,7 +609,7 @@ class PlotlyBackend(Plot):
             elif s.is_geometry:
                 x, y = s.get_data()
                 lkw = dict(
-                    name=s.label, mode="lines", fill="toself", line_color=next(self._cl)
+                    name=s.get_label(self._use_latex), mode="lines", fill="toself", line_color=next(self._cl)
                 )
                 kw = merge({}, lkw, s.rendering_kw)
                 self._fig.add_trace(go.Scatter(x=x, y=y, **kw))
@@ -596,9 +640,13 @@ class PlotlyBackend(Plot):
 
                 elif s.is_2Dline:
                     x, y = self.series[i].get_data()
-                    if s.is_geometry:
-                        self.fig.data[i]["x"] = x
-                    self.fig.data[i]["y"] = y
+                    if not s.is_polar:
+                        if s.is_geometry:
+                            self.fig.data[i]["x"] = x
+                        self.fig.data[i]["y"] = y
+                    else:
+                        self.fig.data[i]["r"] = y
+                        self.fig.data[i]["theta"] = x
 
                 elif s.is_3Dline:
                     x, y, z, param = s.get_data()
@@ -607,15 +655,21 @@ class PlotlyBackend(Plot):
                     self.fig.data[i]["z"] = z
                     self.fig.data[i]["line"]["color"] = param
 
-                elif s.is_3Dsurface and s.is_parametric:
-                    x, y, z = self.series[i].get_data()
-                    self.fig.data[i]["x"] = x
-                    self.fig.data[i]["y"] = y
-                    self.fig.data[i]["z"] = z
+                elif s.is_3Dsurface and (not s.is_domain_coloring) and (not s.is_implicit):
+                    if not s.is_parametric:
+                        x, y, z = s.get_data()
+                        surfacecolor = s.color_func(x, y, z)
+                    else:
+                        x, y, z, u, v = s.get_data()
+                        surfacecolor = s.color_func(x, y, z, u, v)
+                        self.fig.data[i]["x"] = x
+                        self.fig.data[i]["y"] = y
 
-                elif s.is_3Dsurface and (not s.is_domain_coloring):
-                    x, y, z = self.series[i].get_data()
+                    _min, _max = surfacecolor.min(), surfacecolor.max()
                     self.fig.data[i]["z"] = z
+                    self.fig.data[i]["surfacecolor"] = surfacecolor
+                    self.fig.data[i]["cmin"] = _min
+                    self.fig.data[i]["cmax"] = _max
 
                 elif s.is_contour and (not s.is_complex):
                     _, _, zz = s.get_data()
@@ -624,7 +678,10 @@ class PlotlyBackend(Plot):
                 elif s.is_vector and s.is_3D:
                     if s.is_streamlines:
                         raise NotImplementedError
-                    _, _, _, u, v, w = self.series[i].get_data()
+                    x, y, z, u, v, w = self.series[i].get_data()
+                    self.fig.data[i]["x"] = x.flatten()
+                    self.fig.data[i]["y"] = y.flatten()
+                    self.fig.data[i]["z"] = z.flatten()
                     self.fig.data[i]["u"] = u.flatten()
                     self.fig.data[i]["v"] = v.flatten()
                     self.fig.data[i]["w"] = w.flatten()
@@ -636,7 +693,7 @@ class PlotlyBackend(Plot):
                         raise NotImplementedError
                     else:
                         qkw = dict(
-                            line_color=self.quivers_colors[i], scale=0.075, name=s.label
+                            line_color=self.quivers_colors[i], scale=0.075, name=s.get_label(self._use_latex)
                         )
                         kw = merge({}, qkw, s.rendering_kw)
                         quivers = create_quiver(x, y, u, v, **kw)
@@ -701,6 +758,11 @@ class PlotlyBackend(Plot):
                 zeroline=self.grid,  # thick line at x=0
                 scaleanchor="x" if self.aspect == "equal" else None,
             ),
+            polar=dict(
+                angularaxis={'direction': 'counterclockwise', 'rotation': 0},
+                radialaxis={'range': None if not self.ylim else self.ylim},
+                sector=None if not self.xlim else self.xlim
+            ),
             margin=dict(
                 t=50,
                 l=0,
