@@ -28,11 +28,32 @@ from spb.defaults import TWO_D_B, THREE_D_B
 #   create "domain coloring" plots.
 
 
+def _get_labels(args, kwargs):
+    # implement the label keyword argument
+    # NOTE: this function is a workaround until a better integration is
+    # achieved between iplot and all other plotting functions.
+    labels = kwargs.pop("label", [])
+    if (len(labels) > 0) and (len(labels) != len(args)):
+        raise ValueError("The number of labels must be equals to the "
+            "number of expressions being plotted.\nReceived "
+            "{} expressions and {} labels".format(len(args), len(labels)))
+    if len(labels) == 0:
+        labels = [None] * len(args)
+    return labels
+
+
 def _build_series(*args, interactive=False, **kwargs):
     series = []
     # apply the user-specified function to the expression
     #   keys: the user specified keyword arguments
     #   values: [function, label]
+    # NOTE: the label is going to wrap the string representation of the
+    # expression. This design choice precludes the ability of setting latex
+    # labels, but this is not a problem as the user has the ability to set
+    # a custom alias for the function to be plotted. The main motivation for
+    # this choice is that whenever re() or im() is applied to an expression, it
+    # might gets evaluated, resulting in a different expression (something
+    # that the user might not recognize). This design prevents that.
     mapping = {
         "real": [lambda t: re(t), "Re(%s)"],
         "imag": [lambda t: im(t), "Im(%s)"],
@@ -49,9 +70,11 @@ def _build_series(*args, interactive=False, **kwargs):
     if all([hasattr(a, "is_complex") and a.is_complex for a in args]):
         # args is a list of complex numbers
         cls = ComplexPointSeries if not interactive else ComplexPointInteractiveSeries
-        for a in args:
-            # series.append(cls([a], None, str(a), **kwargs))
-            series.append(cls([a], str(a), **kwargs))
+        labels = _get_labels(args, kwargs)
+        for a, lbl in zip(args, labels):
+            if lbl is None:
+                lbl = str(a)
+            series.append(cls([a], lbl, **kwargs))
     elif (
         (len(args) > 0)
         and all([isinstance(a, (list, tuple, Tuple)) for a in args])
@@ -61,8 +84,11 @@ def _build_series(*args, interactive=False, **kwargs):
         # args is a list of tuples of the form (list, label) where list
         # contains complex points
         cls = ComplexPointSeries if not interactive else ComplexPointInteractiveSeries
-        for a in args:
-            series.append(cls(a[0], a[-1], **kwargs))
+        labels = _get_labels(args, kwargs)
+        for a, lbl in zip(args, labels):
+            if lbl is None:
+                lbl = a[-1]
+            series.append(cls(a[0], lbl, **kwargs))
     elif (
         (len(args) > 0)
         and all([isinstance(a, (list, tuple, Tuple)) for a in args])
@@ -70,8 +96,11 @@ def _build_series(*args, interactive=False, **kwargs):
     ):
         # args is a list of lists
         cls = ComplexPointSeries if not interactive else ComplexPointInteractiveSeries
-        for a in args:
-            series.append(cls(a, "", **kwargs))
+        labels = _get_labels(args, kwargs)
+        for a, lbl in zip(args, labels):
+            if lbl is None:
+                lbl = ""
+            series.append(cls(a, lbl, **kwargs))
     else:
         new_args = []
 
@@ -98,13 +127,25 @@ def _build_series(*args, interactive=False, **kwargs):
             for a in tmp:
                 add_series(a)
         else:
-            # plotting a single expression
-            add_series(args)
-
+            if len([t for t in args if isinstance(t, Expr)]) > 1:
+                # multiple expressions with the same range
+                args = list(args)
+                r = None
+                if _is_range(args[-1]):
+                    r = args.pop()
+                for a in args:
+                    a = [a, ] if r is None else [a, r]
+                    add_series(a)
+            else:
+                # plotting a single expression
+                add_series(args)
 
+        labels = _get_labels(new_args, kwargs)
         params = kwargs.get("params", dict())
-        for a in new_args:
+        for a, lbl in zip(new_args, labels):
             expr, ranges, label = a[0], a[1:-1], a[-1]
+            if lbl is not None:
+                label = lbl
 
             # From now on we are dealing with a function of one variable.
             # ranges need to contain complex numbers
@@ -325,7 +366,8 @@ def plot_real_imag(*args, **kwargs):
         Default to True.
 
     line_kw : dict, optional
-        A dictionary of keywords/values which is passed to the backend's function to customize the appearance of the lines. Refer to the
+        A dictionary of keywords/values which is passed to the backend's
+        function to customize the appearance of the lines. Refer to the
         plotting library (backend) manual for more informations.
 
     loss_fn : callable or None
@@ -349,9 +391,10 @@ def plot_real_imag(*args, **kwargs):
         respectively, when `adaptive=False`. For line plots, default to 1000.
         For surface/contour plots (2D and 3D), default to 300.
 
-    n : int, optional
-        Set the same number of discretization points in all directions to be
-        used when `adaptive=False`.
+    n : int or two-elements tuple (n1, n2), optional
+        If an integer is provided, set the same number of discretization
+        points in all directions. If a tuple is provided, it overrides
+        `n1` and `n2`. It only works when `adaptive=False`.
 
     real : boolean, optional
         If True, plot the real part of the complex function. Default to True.
@@ -365,7 +408,8 @@ def plot_real_imag(*args, **kwargs):
         the size will be set by the backend.
 
     surface_kw : dict, optional
-        A dictionary of keywords/values which is passed to the backend's function to customize the appearance of surfaces. Refer to the
+        A dictionary of keywords/values which is passed to the backend's
+        function to customize the appearance of surfaces. Refer to the
         plotting library (backend) manual for more informations.
 
     threed : boolean, optional
@@ -378,6 +422,10 @@ def plot_real_imag(*args, **kwargs):
         If True, a color map highlighting the elevation will be used.
         Default to True.
 
+    use_latex : boolean, optional
+        Turn on/off the rendering of latex labels. If the backend doesn't
+        support latex, it will render the string representations instead.
+
     title : str, optional
         Title of the plot. It is set to the latex representation of
         the expression, if the plot has only one expression.
@@ -587,8 +635,14 @@ def plot_complex(*args, **kwargs):
         A subclass of `Plot`, which will perform the rendering.
         Default to `MatplotlibBackend`.
 
+    label : str or list/tuple, optional
+        The label to be shown in the legend or colorbar in case of a line plot.
+        If not provided, the string representation of `expr` will be used.
+        The number of labels must be  equal to the number of expressions.
+
     line_kw : dict, optional
-        A dictionary of keywords/values which is passed to the backend's function to customize the appearance of lines. Refer to the
+        A dictionary of keywords/values which is passed to the backend's
+        function to customize the appearance of lines. Refer to the
         plotting library (backend) manual for more informations.
 
     loss_fn : callable or None
@@ -612,9 +666,10 @@ def plot_complex(*args, **kwargs):
         respectively, when `adaptive=False`. For line plots, default to 1000.
         For surface/contour plots (2D and 3D), default to 300.
 
-    n : int, optional
-        Set the same number of discretization points in all directions to be
-        used when `adaptive=False`.
+    n : int or two-elements tuple (n1, n2), optional
+        If an integer is provided, set the same number of discretization
+        points in all directions. If a tuple is provided, it overrides
+        `n1` and `n2`. It only works when `adaptive=False`.
 
     show : boolean, optional
         Default to True, in which case the plot will be shown on the screen.
@@ -673,6 +728,10 @@ def plot_complex(*args, **kwargs):
         Title of the plot. It is set to the latex representation of
         the expression, if the plot has only one expression.
 
+    use_latex : boolean, optional
+        Turn on/off the rendering of latex labels. If the backend doesn't
+        support latex, it will render the string representations instead.
+
     xlabel : str, optional
         Label for the x-axis.
 
@@ -718,7 +777,7 @@ def plot_complex(*args, **kwargs):
        :format: doctest
        :include-source: True
 
-       >>> plot_complex((cos(x) + sin(I * x), "f"), (x, -2, 2))
+       >>> plot_complex(cos(x) + sin(I * x), "f", (x, -2, 2))
        Plot object containing:
        [0]: absolute-argument line: cos(x) + I*sinh(x) for x over ((-2+0j), (2+0j))
 
@@ -744,7 +803,7 @@ def plot_complex(*args, **kwargs):
        :include-source: True
 
        >>> plot_complex(gamma(z), (z, -3 - 3*I, 3 + 3*I), threed=True,
-        ...     zlim=(-1, 6))
+       ...     zlim=(-1, 6))
        Plot object containing:
        [0]: cartesian surface: gamma(z) for re(z) over (-3.0, 3.0) and im(z) over (-3.0, 3.0)
 
@@ -814,8 +873,14 @@ def plot_complex_list(*args, **kwargs):
         works if `is_point=True`.
         If False, filled circular markers will be rendered.
 
+    label : str or list/tuple, optional
+        The name associated to the list of the complex numbers to be
+        eventually shown on the legend. The number of labels must be equal to
+        the number of lists.
+
     line_kw : dict, optional
-        A dictionary of keywords/values which is passed to the backend's function to customize the appearance of lines. Refer to the
+        A dictionary of keywords/values which is passed to the backend's
+        function to customize the appearance of lines. Refer to the
         plotting library (backend) manual for more informations.
 
     show : boolean
@@ -830,6 +895,10 @@ def plot_complex_list(*args, **kwargs):
         Title of the plot. It is set to the latex representation of
         the expression, if the plot has only one expression.
 
+    use_latex : boolean, optional
+        Turn on/off the rendering of latex labels. If the backend doesn't
+        support latex, it will render the string representations instead.
+
     xlabel : str, optional
         Label for the x-axis.
 
@@ -955,10 +1024,11 @@ def plot_complex_vector(*args, **kwargs):
         Number of discretization points for the quivers or streamlines in the
         x/y-direction, respectively. Default to 25.
 
-    n : int
-        Set the same number of discretization points in all directions for
-        the quivers or streamlines. It overrides `n1`, `n2`.
-        Default to 25.
+    n : int or two-elements tuple (n1, n2), optional
+        If an integer is provided, set the same number of discretization
+        points in all directions for quivers or streamlines. If a tuple is
+        provided, it overrides `n1` and `n2`. It only works when
+        `adaptive=False`. Default to 25.
 
     nc : int
         Number of discretization points for the scalar contour plot.
@@ -1008,6 +1078,10 @@ def plot_complex_vector(*args, **kwargs):
         Title of the plot. It is set to the latex representation of
         the expression, if the plot has only one expression.
 
+    use_latex : boolean, optional
+        Turn on/off the rendering of latex labels. If the backend doesn't
+        support latex, it will render the string representations instead.
+
     xlabel : str, optional
         Label for the x-axis.
 
