@@ -18,43 +18,102 @@ file_path = os.path.join(cfg_dir, cfg_file)
 def _hardcoded_defaults():
     # Hardcoded default values
     return dict(
-        # Find more Plotly themes at the following page:
-        # https://plotly.com/python/templates/
         plotly={
+            # More themes at: https://plotly.com/python/templates/
             "theme": "seaborn",
-            "grid": True
-        },
-        # Find more Bokeh themes at the following page:
-        # https://docs.bokeh.org/en/latest/docs/reference/themes.html
-        bokeh={
-            "theme": "caliber",
-            "sizing_mode": "stretch_width",
-            "update_event": True,
+            # Show/hide main grid
             "grid": True,
+            # Render latex with Plotly
+            "use_latex": False,
+        },
+        bokeh={
+            # More themes at:
+            # https://docs.bokeh.org/en/latest/docs/reference/themes.html
+            "theme": "caliber",
+            # How will the plot resizes to fill the available space.
+            "sizing_mode": "stretch_width",
+            # Activate/Deactivate automatic update event on panning
+            "update_event": True,
+            # Show/hide main grid
+            "grid": True,
+            # Show/hide minor grid
             "show_minor_grid": True,
+            # Depending on the used Bokeh `themes`, probably need
+            # to adjust the opacity of the minor grid lines
             "minor_grid_line_alpha": 0.6,
+            # Controls the spacing of the dashes in minor grid lines
             "minor_grid_line_dash": [2, 2],
+            # Render latex with Bokeh
+            "use_latex": False,
         },
         k3d={
+            # Background color
             "bg_color": 0xFFFFFF,       # 3620427
+            # Grid color
             "grid_color": 0xE6E6E6,     # 0x888888
+            # Color of the labels
             "label_color": 0x444444,    # 0xDDDDDD
-            "grid": True
+            # Show/hide main grid
+            "grid": True,
+            # Render latex with K3D
+            "use_latex": True,
         },
         matplotlib={
-            "axis_center": None,  # "auto", "center"
+            # Position of the intersection of the axis. If None, use a
+            # standard Matplotlib layout with vertical axis on the left,
+            # horizontal axis on the bottom.
+            # Possible values: "auto", "center", None
+            "axis_center": None,
+            # Show/hide main grid
             "grid": True,
+            # Show/hide minor grid
             "show_minor_grid": True,
-        },
-        backend_2D="matplotlib",
-        backend_3D="matplotlib",
-        complex={
-            "modules": None,     # "mpmath"
-            "coloring": "a"
-        },
-        interactive={
+            # Render latex with Matplotlib
             "use_latex": True,
-            "throttled": False
+        },
+        # Possible values: "matplotlib", "plotly", "bokeh"
+        backend_2D="matplotlib",
+        # Possible values: "matplotlib", "plotly", "k3d"
+        backend_3D="matplotlib",
+
+        # settings about the spb.ccomplex.complex module
+        complex={
+            "modules": None,    # None (default to Numpy/Scipy), "mpmath"
+            "coloring": "a"     # read plot_complex docs for more options
+        },
+
+        # settings about interactive-widget plots
+        interactive={
+            # Render latex on the widget's labels
+            "use_latex": True,
+            # Controls wether sliders trigger the update of `iplot`at each
+            # tick (value False) or only when the mouse click is released
+            # (value True)
+            "throttled": False,
+            # If True, the interactive application will be served on a new
+            # browser window, otherwise it will be shown on Jupyter Notebook
+            "servable": False,
+            # If the interactive application is being served to a new
+            # browser window, an appropriate theme can be choosed.
+            # Possible values: "dark", "light"
+            "theme": "dark"
+        },
+        plot3d={
+            # Wheter to use a color map on a 3D surface
+            "use_cm": False
+        },
+
+        # settings that will be passed to the adaptive library:
+        # https://github.com/python-adaptive/adaptive/
+        adaptive={
+            # higher number produces coarser results
+            "goal": 0.01
+        },
+
+        plot_range={
+            # set the default plot range
+            "min": -10,
+            "max": 10
         }
     )
 
