@@ -1,5 +1,7 @@
-from spb.defaults import TWO_D_B, THREE_D_B
+from spb.defaults import TWO_D_B, THREE_D_B, cfg
+from spb.functions import _set_labels
 from spb.series import (
+    BaseSeries,
     Vector2DSeries,
     Vector3DSeries,
     ContourSeries,
@@ -8,6 +10,7 @@ from spb.series import (
     InteractiveSeries
 )
 from spb.utils import _plot_sympify, _unpack_args, _split_vector, _is_range
+from sympy import MutableDenseMatrix, ImmutableDenseMatrix
 from sympy.core.containers import Tuple
 from sympy.functions.elementary.miscellaneous import sqrt
 from sympy.core.expr import Expr
@@ -108,6 +111,10 @@ def _series(expr, *ranges, label="", interactive=False, **kwargs):
     fs = fs.difference(params.keys())
 
     if split_expr[2] is S.Zero:  # 2D case
+        if isinstance(expr, (MutableDenseMatrix, ImmutableDenseMatrix)) and (label == str(expr)):
+            # in case a Matrix has been provided, but no label
+            # rewrite the label so that it doesn't contain Matrix
+            label = str(split_expr[:-1])
         kwargs = _set_discretization_points(kwargs.copy(), Vector2DSeries)
         if len(fs) > 2:
             raise ValueError(
@@ -130,7 +137,7 @@ def _series(expr, *ranges, label="", interactive=False, **kwargs):
                 )
             ranges = list(ranges)
             for m in missing:
-                ranges.append(Tuple(m, -10, 10))
+                ranges.append(Tuple(m, cfg["plot_range"]["min"], cfg["plot_range"]["max"]))
 
         if len(ranges) > 2:
             raise ValueError("Too many ranges for 2D vector plot.")
@@ -146,6 +153,11 @@ def _series(expr, *ranges, label="", interactive=False, **kwargs):
             InteractiveSeries(split_expr[:2], ranges, label, **kwargs),
         )
     else:  # 3D case
+        if isinstance(expr, (MutableDenseMatrix, ImmutableDenseMatrix)) and (label == str(expr)):
+            # in case a Matrix has been provided, but no label
+            # rewrite the label so that it doesn't contain Matrix
+            label = str(split_expr)
+
         kwargs = _set_discretization_points(kwargs.copy(), Vector3DSeries)
         if len(fs) > 3:
             raise ValueError(
@@ -168,7 +180,7 @@ def _series(expr, *ranges, label="", interactive=False, **kwargs):
                 )
             ranges = list(ranges)
             for m in missing:
-                ranges.append(Tuple(m, -10, 10))
+                ranges.append(Tuple(m, cfg["plot_range"]["min"], cfg["plot_range"]["max"]))
 
         if len(ranges) > 3:
             raise ValueError("Too many ranges for 3D vector plot.")
@@ -189,9 +201,9 @@ def _series(expr, *ranges, label="", interactive=False, **kwargs):
 
         # verify that the slices are of the correct type
         def _check_slice(s):
-            if not isinstance(s, (Expr, Plane)):
+            if not isinstance(s, (Expr, Plane, BaseSeries)):
                 raise ValueError(
-                    "A slice must be of type Plane or Expr.\n"
+                    "A slice must be of type Plane or Expr or BaseSeries.\n"
                     + "Received: {}, {}".format(type(s), s)
                 )
 
@@ -298,14 +310,18 @@ def plot_vector(*args, show=True, **kwargs):
         contour function to customize the appearance. Refer to the plotting
         library (backend) manual for more informations.
 
+    label : str or list/tuple, optional
+        The label to be shown in the colorbar if ``scalar=None``.
+        If not provided, the string representation of `expr` will be used.
+
     n1, n2, n3 : int
         Number of discretization points for the quivers or streamlines in the
         x/y/z-direction, respectively. Default to 25.
 
-    n : int
-        Set the same number of discretization points in all directions for
-        the quivers or streamlines. It overrides `n1`, `n2`, `n3`.
-        Default to 25.
+    n : int or three-elements tuple (n1, n2, n3), optional
+        If an integer is provided, the ranges are sampled uniformly
+        at `n` number of points. If a tuple is provided, it overrides
+        `n1`, `n2` and `n3`. Default to 25.
 
     nc : int
         Number of discretization points for the scalar contour plot.
@@ -345,6 +361,8 @@ def plot_vector(*args, show=True, **kwargs):
 
         - a Plane object from sympy.geometry module.
         - a list of planes.
+        - an instance of ``SurfaceOver2DRangeSeries`` or
+          ``ParametricSurfaceSeries``.
         - a symbolic expression representing a surface of two variables.
 
         The number of discretization points will be `n1`, `n2`, `n3`.
@@ -353,6 +371,8 @@ def plot_vector(*args, show=True, **kwargs):
         - only quivers plots are supported with slices. Streamlines plots
           are unaffected.
         - `n3` will only be used with planes parallel to xz or yz.
+        - `n1`, `n2`, `n3` doesn't affect the slice if it is an instance of
+          ``SurfaceOver2DRangeSeries`` or ``ParametricSurfaceSeries``.
 
     streamlines : boolean
         Whether to plot the vector field using streamlines (True) or quivers
@@ -383,6 +403,10 @@ def plot_vector(*args, show=True, **kwargs):
         Title of the plot. It is set to the latex representation of
         the expression, if the plot has only one expression.
 
+    use_latex : boolean, optional
+        Turn on/off the rendering of latex labels. If the backend doesn't
+        support latex, it will render the string representations instead.
+
     xlabel : str, optional
         Label for the x-axis.
 
@@ -428,15 +452,18 @@ def plot_vector(*args, show=True, **kwargs):
        [0]: contour: sqrt(sin(y)**2 + cos(x)**2) for x over (-3.0, 3.0) and y over (-3.0, 3.0)
        [1]: 2D vector series: [-sin(y), cos(x)] over (x, -3.0, 3.0), (y, -3.0, 3.0)
 
-    Streamlines plot of a 2D vector field with no background scalar field.
+    Streamlines plot of a 2D vector field with no background scalar field and
+    custom label:
 
     .. plot::
        :context: close-figs
        :format: doctest
        :include-source: True
 
-       >>> plot_vector([-sin(y), cos(x)], (x, -3, 3), (y, -3, 3),
-       ...     streamlines=True, scalar=None)
+       >>> expr = [-sin(y), cos(x)]
+       >>> plot_vector(expr, (x, -3, 3), (y, -3, 3),
+       ...     streamlines=True, scalar=None,
+       ...     label="Magnitude of %s" % str(expr))
        Plot object containing:
        [0]: 2D vector series: [-sin(y), cos(x)] over (x, -3.0, 3.0), (y, -3.0, 3.0)
 
@@ -508,6 +535,7 @@ def plot_vector(*args, show=True, **kwargs):
     args = _plot_sympify(args)
     args = _preprocess(*args)
 
+    labels = kwargs.pop("label", [])
     kwargs = _set_discretization_points(kwargs, Vector3DSeries)
     kwargs.setdefault("aspect", "equal")
     kwargs.setdefault("legend", True)
@@ -520,6 +548,7 @@ def plot_vector(*args, show=True, **kwargs):
     else:
         raise ValueError("Mixing 2D vectors with 3D vectors is not allowed.")
 
+    _set_labels(series, labels)
     p = Backend(*series, **kwargs)
     if show:
         p.show()
